<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>LogicNormalizer</title>
</head>

<body>
    <h1>Text Eingabe</h1>
    <form method="post">
        <label for="logic_expression">Logischer Ausdruck:</label>
        <input type="text" id="logic_expression" name="logic_expression"><br>
        <input type="submit" value="Submit">
    </form>
    <h1>Upload</h1>
    <form method=post enctype=multipart/form-data>
        <p><input type=file name=file></p>
        <p><input type=submit value=Upload></p>
    </form>
    <p>
    <table>
        <tr>
            <th>Input</th>
            <td>Disjunktive Normalform</td>
       </tr>
        {% for key, value in result.items() %}
           <tr>
                <th> {{ key }} </th>
                <td> {{ value }} </td>
           </tr>
        {% endfor %}
    </table>
</body>

</html>